@@ -396,15 +396,7 @@ static int dns_res_search(const char *name, int class, int type,
     if (keep_notfound)
 	/* Prepare for returning a null-padded server reply. */
 	memset(answer, 0, anslen);
-    len = res_search(name, class, type, answer, anslen);
-    /* Begin API creep workaround. */
-    if (len < 0 && h_errno == 0) {
-	SET_H_ERRNO(TRY_AGAIN);
-	msg_warn("res_query(\"%s\", %d, %d, %p, %d) returns %d with h_errno==0"
-		 " -- setting h_errno=TRY_AGAIN",
-		 name, class, type, answer, anslen, len);
-    }
-    /* End API creep workaround. */
+    len = res_query(name, class, type, answer, anslen);
     if (len > 0) {
 	SET_H_ERRNO(0);
     } else if (keep_notfound && NOT_FOUND_H_ERRNO(h_errno)) {
@@ -1092,8 +1084,8 @@ int     dns_lookup_rl(const char *name, unsigned flags, DNS_RR **rrlist,
     for (type = va_arg(ap, unsigned); type != 0; type = next) {
 	next = va_arg(ap, unsigned);
 	if (msg_verbose)
-	    msg_info("lookup %s type %s flags %d",
-		     name, dns_strtype(type), flags);
+	    msg_info("lookup %s type %s flags %s",
+		     name, dns_strtype(type), dns_str_resflags(flags));
 	status = dns_lookup_x(name, type, flags, rrlist ? &rr : (DNS_RR **) 0,
 			      fqdn, why, rcode, lflags);
 	if (rrlist && rr)
@@ -1144,8 +1136,8 @@ int     dns_lookup_rv(const char *name, unsigned flags, DNS_RR **rrlist,
     for (type = *types++; type != 0; type = next) {
 	next = *types++;
 	if (msg_verbose)
-	    msg_info("lookup %s type %s flags %d",
-		     name, dns_strtype(type), flags);
+	    msg_info("lookup %s type %s flags %s",
+		     name, dns_strtype(type), dns_str_resflags(flags));
 	status = dns_lookup_x(name, type, flags, rrlist ? &rr : (DNS_RR **) 0,
 			      fqdn, why, rcode, lflags);
 	if (rrlist && rr)
