@@ -13,7 +13,11 @@
 
  /* External interface. */
 
-extern char *split_addr(char *, const char *);
+extern char *split_addr_internal(char *, const char *);
+
+ /* Legacy API. */
+
+#define split_addr	split_addr_internal
 
 /* LICENSE
 /* .ad
