@@ -136,11 +136,6 @@
 /*	Available in Postfix version 2.11 and later:
 /* .IP "\fBtlsmgr_service_name (tlsmgr)\fR"
 /*	The name of the \fBtlsmgr\fR(8) service entry in master.cf.
-/* .PP
-/*	Introduced with Postfix 3.4.6, 3.3.5, 3.2.10, and 3.1.13:
-/* .IP "\fBtls_fast_shutdown_enable (yes)\fR"
-/*	A workaround for implementations that hang Postfix while shuting
-/*	down a TLS session, until Postfix times out.
 /* OBSOLETE STARTTLS SUPPORT CONTROLS
 /* .ad
 /* .fi
@@ -171,8 +166,8 @@
 /* .IP "\fBsyslog_facility (mail)\fR"
 /*	The syslog facility of Postfix logging.
 /* .IP "\fBsyslog_name (see 'postconf -d' output)\fR"
-/*	The mail system name that is prepended to the process name in syslog
-/*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
+/*	A prefix that is prepended to the process name in syslog
+/*	records, so that, for example, "smtpd" becomes "prefix/smtpd".
 /* SEE ALSO
 /*	postscreen(8), Postfix zombie blocker
 /*	smtpd(8), Postfix SMTP server
@@ -506,8 +501,9 @@ static void tlsp_strategy(TLSP_STATE *state)
     if (NBBIO_ERROR_FLAGS(plaintext_buf)) {
 	if (NBBIO_ACTIVE_FLAGS(plaintext_buf))
 	    nbbio_disable_readwrite(state->plaintext_buf);
-	if (!SSL_in_init(tls_context->con)
-	    && (ssl_stat = SSL_shutdown(tls_context->con)) < 0) {
+	ssl_stat = SSL_shutdown(tls_context->con);
+	/* XXX Wait for return value 1 if sessions are to be reused? */
+	if (ssl_stat < 0) {
 	    handshake_err = SSL_get_error(tls_context->con, ssl_stat);
 	    tlsp_eval_tls_error(state, handshake_err);
 	    /* At this point, state could be a dangling pointer. */
@@ -663,7 +659,7 @@ static void tlsp_ciphertext_event(int event, void *context)
 
 /* tlsp_start_tls - turn on TLS or force disconnect */
 
-static int tlsp_start_tls(TLSP_STATE *state)
+static void tlsp_start_tls(TLSP_STATE *state)
 {
     TLS_SERVER_START_PROPS props;
     static char *cipher_grade;
@@ -676,7 +672,7 @@ static int tlsp_start_tls(TLSP_STATE *state)
      */
 
     /*
-     * Perform the before-handshake portion of the per-session initalization.
+     * Perform the before-handshake portion of the per-session initialization.
      * Pass a null VSTREAM to indicate that this program, will do the
      * ciphertext I/O, not libtls.
      * 
@@ -716,7 +712,7 @@ static int tlsp_start_tls(TLSP_STATE *state)
 
     if (state->tls_context == 0) {
 	tlsp_state_free(state);
-	return (-1);
+	return;
     }
 
     /*
@@ -729,7 +725,6 @@ static int tlsp_start_tls(TLSP_STATE *state)
      * XXX Do we care about certificate verification results? Not as long as
      * postscreen(8) doesn't actually receive email.
      */
-    return (0);
 }
 
 /* tlsp_get_fd_event - receive final postscreen(8) hand-off information */
@@ -777,8 +772,7 @@ static void tlsp_get_fd_event(int event, void *context)
      * Perform the TLS layer before-handshake initialization. We perform the
      * remainder after the TLS handshake completes.
      */
-    if (tlsp_start_tls(state) < 0)
-	return;
+    tlsp_start_tls(state);
 
     /*
      * Trigger the initial proxy server I/Os.
