@@ -139,6 +139,9 @@
 /*	length would be exceeded, the program simulates a memory
 /*	allocation problem (i.e. it terminates through msg_fatal()).
 /*	This fuctionality is currently unimplemented.
+/* .IP "CA_VSTRING_CTL_EXACT (no argument)"
+/*	Allocate the requested amounts, instead of rounding up.
+/*	This should be used for tests only.
 /* .IP "CA_VSTRING_CTL_END (no argument)"
 /*	Specifies the end of the argument list. Forgetting to terminate
 /*	the argument list may cause the program to crash.
@@ -263,6 +266,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System libraries. */
@@ -280,10 +288,6 @@
 #include "vbuf_print.h"
 #include "vstring.h"
 
-#ifndef SSIZE_T_MAX
-#define SSIZE_T_MAX __MAXINT__(ssize_t)
-#endif
-
 /* vstring_extend - variable-length string buffer extension policy */
 
 static void vstring_extend(VBUF *bp, ssize_t incr)
@@ -300,16 +304,13 @@ static void vstring_extend(VBUF *bp, ssize_t incr)
      * 
      * The length overflow tests here and in vstring_alloc() should protect us
      * against all length overflow problems within vstring library routines.
-     * (The tests are redundant as long as mymalloc() and myrealloc() reject
-     * negative length parameters).
      */
-    if (bp->len > incr)
+    if ((bp->flags & VSTRING_FLAG_EXACT) == 0 && bp->len > incr)
 	incr = bp->len;
-    if (bp->len > SSIZE_T_MAX - incr - 1)
+    if (bp->len > SSIZE_T_MAX - incr)
 	msg_fatal("vstring_extend: length overflow");
     new_len = bp->len + incr;
-    bp->data = (unsigned char *) myrealloc((void *) bp->data, new_len + 1);
-    bp->data[new_len] = 0;
+    bp->data = (unsigned char *) myrealloc((void *) bp->data, new_len);
     bp->len = new_len;
     bp->ptr = bp->data + used;
     bp->cnt = bp->len - used;
@@ -349,13 +350,12 @@ VSTRING *vstring_alloc(ssize_t len)
 {
     VSTRING *vp;
 
-    if (len < 1 || len > SSIZE_T_MAX - 1)
+    if (len < 1)
 	msg_panic("vstring_alloc: bad length %ld", (long) len);
     vp = (VSTRING *) mymalloc(sizeof(*vp));
     vp->vbuf.flags = 0;
     vp->vbuf.len = 0;
-    vp->vbuf.data = (unsigned char *) mymalloc(len + 1);
-    vp->vbuf.data[len] = 0;
+    vp->vbuf.data = (unsigned char *) mymalloc(len);
     vp->vbuf.len = len;
     VSTRING_RESET(vp);
     vp->vbuf.data[0] = 0;
@@ -393,6 +393,9 @@ void    vstring_ctl(VSTRING *vp,...)
 	    if (vp->maxlen < 0)
 		msg_panic("vstring_ctl: bad max length %ld", (long) vp->maxlen);
 	    break;
+	case VSTRING_CTL_EXACT:
+	    vp->vbuf.flags |= VSTRING_FLAG_EXACT;
+	    break;
 	}
     }
     va_end(ap);
